<?php
/**
 * CleverReach WooCommerce Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WooCommerce\Components\Repositories\Customers;

/**
 * Class Contact_Repository
 *
 * @package CleverReach\WooCommerce\Components\Repositories\Customers
 */
class Contact_Repository extends Base_Receiver_Repository {

	/**
	 * Get receiver by email.
	 *
	 * @param string $email Receiver email.
	 *
	 * @inheritDoc
	 */
	public function get_by_email( $email ) {
		$recipients = $this->get_by_emails( array( $email ) );

		return ! empty( $recipients ) ? current( $recipients ) : null;
	}

	/**
	 * Retrieves list of receivers identified by the provided emails.
	 *
	 * @param array $emails List of emails.
	 *
	 * @inheritDoc
	 */
	public function get_by_emails( $emails ) {
		$prepared_emails = implode( "', '", $emails );
		$meta_keys       = implode( "', '", $this->get_registered_user_meta_keys() );

		$blog_prefix = $this->get_db()->get_blog_prefix( get_current_blog_id() );

		$sql_query = "select u.ID, u.user_email, u.user_registered, meta.meta_key, meta.meta_value
					  from {$this->get_db()->users} u
					      join {$this->get_db()->usermeta} meta_capabilities on u.ID = meta_capabilities.user_id
					      join (select user_meta.user_id, user_meta.meta_key, user_meta.meta_value
					      		from {$this->get_db()->usermeta} user_meta
					      		where user_meta.meta_key in ('$meta_keys')) as meta on u.ID = meta.user_id
					  where u.user_email in ('$prepared_emails')
					    and meta_capabilities.meta_key = '{$blog_prefix}capabilities'
					    and u.ID not in (select order_cu.meta_value
										 from {$this->get_db()->postmeta} order_cu
					    				 where order_cu.meta_key = '_customer_user')
					    and u.user_email not in (select order_be.meta_value
                           						from {$this->get_db()->postmeta} order_be
                           						    join {$this->get_db()->postmeta} order_cu on order_be.post_id = order_cu.post_id
					    						where order_be.meta_key = '_billing_email'
					                             and order_cu.meta_key = '_customer_user'
					                             and order_cu.meta_value = 0)
					    and u.ID not in (select subscriber_meta.user_id
                                         from {$this->get_db()->usermeta} subscriber_meta
                                         	join {$this->get_db()->usermeta} capabilities on subscriber_meta.user_id = capabilities.user_id
                                         where subscriber_meta.meta_key = '" . Subscriber_Repository::get_newsletter_column() . "'
                                            and subscriber_meta.meta_value = '" . Subscriber_Repository::STATUS_SUBSCRIBED . "'
                                            and capabilities.meta_key = '{$blog_prefix}capabilities')				                          
					  order by u.user_email";

		$results = $this->get_db()->get_results( $sql_query, ARRAY_A );

		if ( empty( $results ) ) {
			return array();
		}

		return $this->sanitize( $results, self::CUSTOMER_ID_PREFIX );
	}

	/**
	 *  Retrieves list of receiver emails provided by the integration.
	 *
	 * @inheritDoc
	 */
	public function get_emails() {
		$blog_prefix = $this->get_db()->get_blog_prefix( get_current_blog_id() );

		$sql_query = "select u.user_email as email
					  from {$this->get_db()->users} u
						  join {$this->get_db()->usermeta} meta_capabilities on u.ID = meta_capabilities.user_id
					  where meta_capabilities.meta_key = '{$blog_prefix}capabilities'
					    and u.ID not in (select order_cu.meta_value
                           from {$this->get_db()->postmeta} order_cu
                           where order_cu.meta_key = '_customer_user')
					  	and u.user_email not in (select order_be.meta_value
                           from {$this->get_db()->postmeta} order_be
                                    join {$this->get_db()->postmeta} order_cu on order_be.post_id = order_cu.post_id
                           where order_be.meta_key = '_billing_email'
                             and order_cu.meta_key = '_customer_user'
                             and order_cu.meta_value = 0)
                        and u.ID not in (select subscriber.ID
                                         from {$this->get_db()->users} subscriber
                                            join {$this->get_db()->usermeta} meta_cr_status on subscriber.ID = meta_cr_status.user_id
                                         where meta_cr_status.meta_key = '" . Subscriber_Repository::get_newsletter_column() . "'
                                            and meta_cr_status.meta_value = '" . Subscriber_Repository::STATUS_SUBSCRIBED . "')
                      group by u.user_email";

		$results = $this->get_db()->get_results( $sql_query, ARRAY_A );

		if ( empty( $results ) ) {
			return array();
		}

		return array_map(
			function ( $res ) {
				return $res['email'];
			},
			$results
		);
	}

}
