<?php

namespace CleverReach\WooCommerce\IntegrationCore\BusinessLogic\WebHookEvent\Tasks\SubTasks;

use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\Group\Contracts\GroupService;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\WebHookEvent\Http\Proxy;
use CleverReach\WooCommerce\IntegrationCore\BusinessLogic\WebHookEvent\Tasks\Contracts\ExecutionContextAware;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WooCommerce\IntegrationCore\Infrastructure\TaskExecution\Task;

abstract class SubTask extends Task implements ExecutionContextAware
{
    /**
     * @var callable
     */
    private $contextProvider;

    public function setExecutionContextProvider(callable $provider)
    {
        $this->contextProvider = $provider;
    }

    /**
     * Retrieves execution context.
     *
     * @return \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\WebHookEvent\Tasks\Context\ExecutionContext
     */
    protected function getExecutionContext()
    {
        return call_user_func($this->contextProvider);
    }

    /**
     * Retrieves events service.
     *
     * @return \CleverReach\WooCommerce\IntegrationCore\BusinessLogic\WebHookEvent\Contracts\EventsService | object
     */
    protected function getEventsService()
    {
        return ServiceRegister::getService($this->getExecutionContext()->getEventServiceClass());
    }

    /**
     * Retrieves events proxy.
     *
     * @return Proxy | object
     */
    protected function getEventsProxy()
    {
        return ServiceRegister::getService(Proxy::CLASS_NAME);
    }

    /**
     * Provides group service.
     *
     * @return GroupService | object
     */
    protected function getGroupService()
    {
        return ServiceRegister::getService(GroupService::CLASS_NAME);
    }
}